﻿using System;
using System.Windows.Media;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using ff14bot.Managers;
using MultiBot.Settings;
using TreeSharp;
using System.Linq;
using Action = TreeSharp.Action;
using System.Threading;

namespace MultiBot
{
    public class MutliBot : IBotPlugin
    {
        #region Necessary Stuff
        public string Author { get { return "Exmortem"; } }
        public Version Version { get { return new Version(1, 0); } }
        public string Name { get { return "MultiBot"; } }
        public string Description { get { return "Runs Fate Bot in conjunction with PvP bot."; } }
        public bool WantButton
        {
            get { return true; }
        }
        public string ButtonText
        {
            get { return "MultiBot"; }
        }
        public void OnButtonPress()
        {
            if (_form == null || _form.IsDisposed || _form.Disposing)
                _form = new MultiBotForm();

            _form.ShowDialog();  
        }
        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }
        #endregion

        private Composite Root;
        public static MultiBotSettings settings = MultiBotSettings.instance;
        private MultiBotForm _form;

        #region Init
        public void OnInitialize()
        {

        }
        public void OnShutdown()
        {

        }
        public void OnEnabled()
        {
            Logging.Write(Colors.Gold, "[MultiBot] Enabled");

            Root = CreateBehaviorLogic();
            Root.Start(null);
        }
        public void OnDisabled()
        {
            Logging.Write(Colors.Gold, "[MultiBot] Disabled");

            Root.Stop(null);
            Root = null;

        }
        #endregion

        private static Composite CreateBehaviorLogic()
        {
            return new PrioritySelector(
                
                new Decorator(r => WorldManager.ZoneId == 337 && BotManager.Current.Name == "PvP", new Action(r => new ActionAlwaysSucceed())),

                #region PvP
                new Decorator(r => BotManager.Current.Name == "PvP",
                    new PrioritySelector(
                        new Decorator(r => DutyManager.InQueue && WorldManager.ZoneId != 337,
                            new Action(r => ChangeBot(settings.WhichBotToRun)))

                         )),
                #endregion

                new Decorator(r => BotManager.Current.Name == settings.WhichBotToRun,
                    new PrioritySelector(
                        new Decorator(r => (!DutyManager.InQueue || DutyManager.DutyReady) && !Core.Player.InCombat,
                            new Action(r => ChangeBot("PvP")))
                        
                        ))
            );
        }

        public static void ChangeBot(string bot)
        {
            ThreadPool.QueueUserWorkItem(o =>
            {
                TreeRoot.Stop(); 
                Thread.Sleep(1500); 
                BotManager.SetCurrent(BotManager.Bots.FirstOrDefault(u => u.Name.Contains(bot)));
                Logging.Write(Colors.Gold, "[MultiBot] Changed BotBase to " + bot);
                TreeRoot.Start();
            });
        }

        #region Pulse
        public void OnPulse()
        {
            try
            {
                Root.Tick(null);
                if (Root.LastStatus != RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                Logging.WriteException(e);
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }
        #endregion
    }
}